package com.logicbig.example;

import org.primefaces.model.menu.DefaultMenuItem;
import org.primefaces.model.menu.DefaultMenuModel;
import org.primefaces.model.menu.MenuModel;

import javax.annotation.PostConstruct;
import javax.faces.bean.ManagedBean;
import javax.faces.bean.ManagedProperty;
import javax.faces.bean.RequestScoped;
import javax.faces.context.FacesContext;
import javax.servlet.http.HttpServletRequest;
import java.util.List;

@ManagedBean(name = "empBean")
@RequestScoped
public class EmployeeViewBean {

    @ManagedProperty("#{param.viewType}")
    private String viewTypeParam;

    private ViewType viewType;

    @PostConstruct
    public void init() {
        if (viewTypeParam != null) {
            viewType = ViewType.valueOf(viewTypeParam);
        } else {
            viewType = ViewType.Main;
        }
    }

    public void setViewTypeParam(String viewTypeParam) {
        this.viewTypeParam = viewTypeParam;
    }

    public MenuModel getBreadCrumbModel() {
        if (viewType == null || viewType == ViewType.Main) {
            return null;
        }
        MenuModel model = new DefaultMenuModel();
        addItemToMenuModel(model, "Home", "/index.xhtml", false);

        switch (viewType) {
            case DeptList:
                addItemToMenuModel(model, "Departments", null, true);
                break;
            case EmpList:
                addItemToMenuModel(model, "Employees", null, true);
                break;
            case Dept:
                addItemToMenuModel(model, "Departments",
                        createLink(ViewType.DeptList), false);
                addItemToMenuModel(model, getDepartmentById().getName(), null, true);
                break;
            case Emp:
                addItemToMenuModel(model, "Employees", createLink(ViewType.EmpList),
                        false);
                addItemToMenuModel(model, getEmployeeById().getName(), null, true);
                break;
        }

        return model;
    }

    private String createLink(ViewType viewType) {
        return String.format("index.xhtml?viewType=%s", viewType);
    }

    private void addItemToMenuModel(MenuModel model, String value, String url, boolean disabled) {
        DefaultMenuItem element = new DefaultMenuItem();
        element.setValue(value);
        element.setUrl(url);
        element.setDisabled(disabled);
        model.addElement(element);
    }

    public ViewType getViewType() {
        return viewType;
    }

    public List<Employee> getEmployeeList() {
        return EmployeeService.Instance.getEmployeeList();
    }

    public List<Department> getDepartmentList() {
        return EmployeeService.Instance.getDepartmentList();
    }

    public Employee getEmployeeById() {
        long id = extractId("empId");
        return EmployeeService.Instance.findEmployeeById(id);
    }

    public Department getDepartmentById() {
        long id = extractId("deptId");
        return EmployeeService.Instance.findDepartmentById(id);
    }

    public long extractId(String idVar) {
        HttpServletRequest request = (HttpServletRequest) FacesContext.getCurrentInstance()
                                                                      .getExternalContext()
                                                                      .getRequest();
        String deptId = request.getParameter(idVar);
        if (deptId == null) {
            return -1;
        }
        return Long.parseLong(deptId);
    }

    public enum ViewType {
        Main,
        EmpList,
        DeptList,
        Emp,
        Dept;
    }
}